@@ -3,6 +3,7 @@ package Jifty::DBI::Record;
 use strict;
 use warnings;
 
+use vars qw($AUTOLOAD);
 use Class::ReturnValue  ();
 use Lingua::EN::Inflect ();
 use Jifty::DBI::Column  ();
@@ -15,11 +16,8 @@ use base qw/
 
 our $VERSION = '0.01';
 
-Jifty::DBI::Record->mk_classdata(qw/COLUMNS/);
-Jifty::DBI::Record->mk_classdata(qw/TABLE_NAME/ );
-Jifty::DBI::Record->mk_classdata(qw/_READABLE_COLS_CACHE/);
-Jifty::DBI::Record->mk_classdata(qw/_WRITABLE_COLS_CACHE/);
-Jifty::DBI::Record->mk_classdata(qw/_COLUMNS_CACHE/ );
+
+Jifty::DBI::Record->mk_classdata('COLUMNS');
 
 =head1 NAME
 
@@ -29,24 +27,18 @@ Jifty::DBI::Record - Superclass for records loaded by Jifty::DBI::Collection
 
   package MyRecord;
   use base qw/Jifty::DBI::Record/;
-
+  
 =head1 DESCRIPTION
 
-Jifty::DBI::Record encapuslates records and tables as part of the L<Jifty::DBI>
+Jifty::DBI::Record encapuslates records and tables as part of the L<Jifty::DBI> 
 object-relational mapper.
 
 =head1 METHODS
 
-=head2 new ARGS
+=head2 new 
 
 Instantiate a new, empty record object.
 
-ARGS is a hash used to pass parameters to the C<_init()> function.
-
-Unless it is overloaded, the _init() function expects one key of
-'handle' with a value containing a reference to a Jifty::DBI::Handle
-object.
-
 =cut
 
 sub new {
@@ -59,12 +51,7 @@ sub new {
     $self->_init_columns() unless $self->COLUMNS;
     $self->input_filters('Jifty::DBI::Filter::Truncate');
 
-    if ( scalar(@_) == 1 ) {
-        Carp::cluck("new(\$handle) is deprecated, use new( handle => \$handle )");
-        $self->_init( handle => shift );
-    } else {
-        $self->_init(@_);
-    }
+    $self->_init(@_);
 
     return $self;
 }
@@ -72,27 +59,8 @@ sub new {
 # Not yet documented here.  Should almost certainly be overloaded.
 sub _init {
     my $self   = shift;
-    my %args   = (@_);
-    if ( $args{'handle'} ) {
-        $self->_handle( $args{'handle'} );
-    }
-
-}
-
-sub import {
-    my $class = shift;
-    my ($flag) = @_;
-    if ($class->isa(__PACKAGE__) and defined $flag and $flag eq '-base') {
-        my $descendant = (caller)[0];
-        no strict 'refs';
-        push @{$descendant . '::ISA'}, $class;
-        shift;
-
-        # run the schema callback
-        my $callback = shift;
-        $callback->() if $callback;
-    }
-    $class->SUPER::import(@_);
+    my $handle = shift;
+    $self->_handle($handle);
 }
 
 =head2 id
@@ -120,6 +88,116 @@ sub primary_keys {
     return (%hash);
 }
 
+sub DESTROY {
+    return 1;
+}
+
+sub AUTOLOAD {
+    my $self = $_[0];
+
+    $self->_init_columns() unless $self->COLUMNS;
+
+    my ( $column_name, $action ) = $self->_parse_autoload_method($AUTOLOAD);
+
+    unless ( $action and $column_name ) {
+        my ( $package, $filename, $line ) = caller;
+        die "$AUTOLOAD Unimplemented in $package. ($filename line $line) \n";
+    }
+
+    my $column = $self->column($column_name);
+
+    unless ($column) {
+        my ( $package, $filename, $line ) = caller;
+        die "$AUTOLOAD Unimplemented in $package. ($filename line $line) \n";
+    }
+
+    no strict 'refs';    # We're going to be defining subs
+    if ( $action eq 'read' ) {
+        return '' unless $column->readable;
+
+        if ( UNIVERSAL::isa( $column->refers_to, "Jifty::DBI::Record" ) ) {
+            *{$AUTOLOAD} = sub {
+                $_[0]->_to_record( $column_name,
+                    $_[0]->__value($column_name) );
+            };
+        }
+        elsif (
+            UNIVERSAL::isa( $column->refers_to, "Jifty::DBI::Collection" ) )
+        {
+            *{$AUTOLOAD} = sub { $_[0]->_collection_value($column_name) };
+        }
+        else {
+            *{$AUTOLOAD} = sub { return ( $_[0]->_value($column_name) ) };
+        }
+        goto &$AUTOLOAD;
+    }
+    elsif ( $action eq 'write' ) {
+        return ( 0, 'Immutable column' ) unless $column->writable;
+
+        if ( UNIVERSAL::isa( $column->refers_to, "Jifty::DBI::Record" ) ) {
+            *{$AUTOLOAD} = sub {
+                my $self = shift;
+                my $val  = shift;
+
+                $val = $val->id
+                    if UNIVERSAL::isa( $val, 'Jifty::DBI::Record' );
+                return (
+                    $self->_set( column => $column_name, value => $val ) );
+            };
+        }
+        else {
+            *{$AUTOLOAD} = sub {
+                return (
+                    $_[0]->_set( column => $column_name, value => $_[1] ) );
+            };
+        }
+        goto &$AUTOLOAD;
+    }
+    elsif ( $action eq 'validate' ) {
+        *{$AUTOLOAD}
+            = sub { return ( $_[0]->_validate( $column_name, $_[1] ) ) };
+        goto &$AUTOLOAD;
+    }
+
+    else {
+        my ( $package, $filename, $line ) = caller;
+        die "$AUTOLOAD Unimplemented in $package. ($filename line $line) \n";
+    }
+
+}
+
+=head2 _parse_autoload_method $AUTOLOAD
+
+Parses autoload methods and attempts to determine if they're 
+set, get or validate calls.
+
+Returns a tuple of (COLUMN_NAME, ACTION);
+
+=cut
+
+sub _parse_autoload_method {
+    my $self   = shift;
+    my $method = shift;
+
+    my ( $column_name, $action );
+
+    if ( $method =~ /^.*::set_(\w+)$/o ) {
+        $column_name = $1;
+        $action      = 'write';
+    }
+    elsif ( $method =~ /^.*::validate_(\w+)$/o ) {
+        $column_name = $1;
+        $action      = 'validate';
+
+    }
+    elsif ( $method =~ /^.*::(\w+)$/o ) {
+        $column_name = $1;
+        $action      = 'read';
+
+    }
+    return ( $column_name, $action );
+
+}
 
 =head2 _accessible COLUMN ATTRIBUTE
 
@@ -138,6 +216,7 @@ sub _accessible {
     my $self        = shift;
     my $column_name = shift;
     my $attribute   = lc( shift || '' );
+
     my $col = $self->column($column_name);
     return undef unless ( $col and $col->can($attribute) );
     return $col->$attribute();
@@ -183,105 +262,9 @@ sub _init_columns {
         $column->readable(1);
         $column->type('serial');
         $column->mandatory(1);
-
-        $self->_init_methods_for_column($column);
     }
 }
 
-sub _init_methods_for_column {
-    my $self   = $_[0];
-    my $column = $_[1];
-    my $column_name
-        = ( $column->aliased_as ? $column->aliased_as : $column->name );
-    my $package = ref($self) || $self;
-
-    # Make sure column has a record_class set as not all columns are added
-    # through add_column
-    $column->record_class( $package ) if not $column->record_class;
-
-    no strict 'refs';    # We're going to be defining subs
-
-    if ( not $self->can($column_name) ) {
-        # Accessor
-        my $subref;
-        if ( $column->readable ) {
-            if ( UNIVERSAL::isa( $column->refers_to, "Jifty::DBI::Record" ) )
-            {
-                $subref = sub {
-                    $_[0]->_to_record( $column_name,
-                        $_[0]->__value($column_name) );
-                };
-            } elsif (
-                UNIVERSAL::isa(
-                    $column->refers_to, "Jifty::DBI::Collection"
-                )
-                )
-            {
-                $subref = sub { $_[0]->_collection_value($column_name) };
-            } else {
-                $subref = sub { return ( $_[0]->_value($column_name) ) };
-            }
-        } else {
-            $subref = sub { return '' }
-        }
-        *{ $package . "::" . $column_name } = $subref;
-
-    }
-
-    if ( not $self->can( "set_" . $column_name ) ) {
-        # Mutator
-        my $subref;
-        if ( $column->writable ) {
-            if ( UNIVERSAL::isa( $column->refers_to, "Jifty::DBI::Record" ) )
-            {
-                $subref = sub {
-                    my $self = shift;
-                    my $val  = shift;
-
-                    $val = $val->id
-                        if UNIVERSAL::isa( $val, 'Jifty::DBI::Record' );
-                    return (
-                        $self->_set( column => $column_name, value => $val )
-                    );
-                };
-            } elsif (
-                UNIVERSAL::isa(
-                    $column->refers_to, "Jifty::DBI::Collection"
-                )
-                )
-            {    # XXX elw: collections land here, now what?
-                my $ret     = Class::ReturnValue->new();
-                my $message = "Collection column '$column_name' not writable";
-                $ret->as_array( 0, $message );
-                $ret->as_error(
-                    errno        => 3,
-                    do_backtrace => 0,
-                    message      => $message
-                );
-                $subref = sub { return ( $ret->return_value ); };
-            } else {
-                $subref = sub {
-                    return (
-                        $_[0]->_set( column => $column_name, value => $_[1] )
-                    );
-                };
-            }
-        } else {
-            my $ret     = Class::ReturnValue->new();
-            my $message = 'Immutable column';
-            $ret->as_array( 0, $message );
-            $ret->as_error(
-                errno        => 3,
-                do_backtrace => 0,
-                message      => $message
-            );
-            $subref = sub { return ( $ret->return_value ); };
-        }
-        *{ $package . "::" . "set_" . $column_name } = $subref;
-    }
-}
-
-
 =head2 _to_record COLUMN VALUE
 
 This B<PRIVATE> method takes a column name and a value for that column. 
@@ -299,8 +282,8 @@ sub _to_record {
     my $column_name = shift;
     my $value       = shift;
 
-    my $column        = $self->column($column_name);
-    my $classname     = $column->refers_to();
+    my $column    = $self->column($column_name);
+    my $classname = $column->refers_to();
     my $remote_column = $column->by() || 'id';
 
     return       unless defined $value;
@@ -309,8 +292,8 @@ sub _to_record {
 
     # XXX TODO FIXME we need to figure out the right way to call new here
     # perhaps the handle should have an initiializer for records/collections
-    my $object = $classname->new( handle => $self->_handle );
-    $object->load_by_cols( $remote_column => $value );
+    my $object = $classname->new( $self->_handle );
+    $object->load_by_cols($remote_column => $value );
     return $object;
 }
 
@@ -326,28 +309,11 @@ sub _collection_value {
     return undef unless $classname;
     return unless UNIVERSAL::isa( $classname, 'Jifty::DBI::Collection' );
 
-    if ( my $prefetched_col = $self->_prefetched_collection($method_name)) {
-        return $prefetched_col;
-    }
-
-    use Devel::SimpleTrace;
     my $coll = $classname->new( handle => $self->_handle );
-    $coll->limit( column => $column->by(), value => $self->id );
+    $coll->limit( column =>  $column->by(), value => $self->id );
     return $coll;
 }
 
-sub _prefetched_collection {
-    my $self =shift;
-    my $column_name = shift;
-    if (@_) {
-        $self->{'_prefetched_collections'}->{$column_name} = shift;
-    } else {
-        return $self->{'_prefetched_collections'}->{$column_name};
-    }
-
-}
-
-
 =head2 add_column
 
 =cut
@@ -356,58 +322,29 @@ sub add_column {
     my $self = shift;
     my $name = shift;
     $name = lc $name;
-    
     $self->COLUMNS->{$name} = Jifty::DBI::Column->new()
-    unless exists $self->COLUMNS->{$name};
-    $self->_READABLE_COLS_CACHE(undef);
-    $self->_WRITABLE_COLS_CACHE(undef);
-    $self->_COLUMNS_CACHE(undef );
+        unless exists $self->COLUMNS->{$name};
     $self->COLUMNS->{$name}->name($name);
-
-    my $class = ref( $self ) || $self;
-    $self->COLUMNS->{$name}->record_class( $class );
-
     return $self->COLUMNS->{$name};
 }
 
 =head2 column
 
-    my $value = $self->column($column);
-
-Returns the $value of a $column.
-
 =cut
 
 sub column {
     my $self = shift;
     my $name = lc( shift || '' );
-    my $col = $self->COLUMNS;
-    return undef unless $col && exists $col->{$name};
-    return $col->{$name};
+    return undef unless $self->COLUMNS and $self->COLUMNS->{$name};
+    return $self->COLUMNS->{$name};
 
 }
 
-=head2 columns
-
-    my @columns = $record->columns;
-
-Returns a sorted list of a $record's @columns.
-
-=cut
-
 sub columns {
     my $self = shift;
-    return @{$self->_COLUMNS_CACHE() || $self->_COLUMNS_CACHE([
-        sort {
-            ( ( ( $b->type || '' ) eq 'serial' )
-                <=> ( ( $a->type || '' ) eq 'serial' ) )
-                or ( ($a->sort_order || 0) <=> ($b->sort_order || 0))
-                or ( $a->name cmp $b->name )
-            } values %{ $self->COLUMNS }
-
-
-	])}
-
+    return ( sort { 
+         ((($b->type||'') eq 'serial') <=> (($a->type||'') eq 'serial')) or 
+            ( $a->name cmp $b->name) } values %{ $self->COLUMNS } );
 }
 
 # sub {{{ readable_attributes
@@ -420,7 +357,7 @@ Returns a list this table's readable columns
 
 sub readable_attributes {
     my $self = shift;
-    return @{$self->_READABLE_COLS_CACHE() || $self->_READABLE_COLS_CACHE([sort map { $_->name } grep { $_->readable } $self->columns])};
+    return sort map { $_->name } grep { $_->readable } $self->columns;
 }
 
 =head2 writable_attributes
@@ -432,81 +369,60 @@ Returns a list of this table's writable columns
 
 sub writable_attributes {
     my $self = shift;
-    return @{$self->_WRITABLE_COLS_CACHE() || $self->_WRITABLE_COLS_CACHE([sort map { $_->name } grep { $_->writable } $self->columns])};
+    return sort map { $_->name } grep { $_->writable } $self->columns;
 }
 
-=head2 record values
-
-As you've probably already noticed, C<Jifty::DBI::Record> autocreates methods for your
-standard get/set accessors. It also provides you with some hooks to massage the values
-being loaded or stored.
-
-When you fetch a record value by calling C<$my_record-E<gt>some_field>, C<Jifty::DBI::Record>
-provides the following hook
-
-=over
-
-
-
-=item after_I<column_name>
-
-This hook is called with a reference to the value returned by
-Jifty::DBI. Its return value is discarded.
-
-=back
-
-When you set a value, C<Jifty::DBI> provides the following hooks
-
-=over
-
-=item before_set_I<column_name> PARAMHASH
-
-C<Jifty::DBI::Record> passes this function a reference to a paramhash
-composed of:
-
-=over
-
-=item column
-
-The name of the column we're updating.
-
-=item value
-
-The new value for I<column>.
-
-=item is_sql_function
-
-A boolean that, if true, indicates that I<value> is an SQL function,
-not just a value.
-
-=back
-
-If before_set_I<column_name> returns false, the new value isn't set.
-
-=item after_set_I<column_name> PARAMHASH
-
-This hook will be called after a value is successfully set in the
-database. It will be called with a reference to a paramhash that
-contains C<column> and C<value> keys. If C<value> was a SQL function,
-it will now contain the actual value that was set.
-
-This hook's return value is ignored.
-
-=item validate_I<column_name> VALUE
-
-This hook is called just before updating the database. It expects the
-actual new value you're trying to set I<column_name> to. It returns
-two values.  The first is a boolean with truth indicating success. The
-second is an optional message. Note that validate_I<column_name> may be
-called outside the context of a I<set> operation to validate a potential
-value. (The Jifty application framework uses this as part of its AJAX
-validation system.)
-
-=back
+=head2 __value
 
+Takes a column name and returns that column's value. Subclasses should
+never override __value.
 
 =cut
 
+sub __value {
+    my $self        = shift;
+    my $column_name = shift;
+
+    # If the requested column is actually an alias for another, resolve it.
+    if ( $self->column($column_name)
+        and defined $self->column($column_name)->alias_for_column )
+    {
+        $column_name = $self->column($column_name)->alias_for_column();
+    }
+
+    my $column = $self->column($column_name);
+
+    return unless ($column);
+
+    #Carp::confess unless ($column);
+
+    if ( !$self->{'fetched'}{ $column->name } and my $id = $self->id() ) {
+        my $pkey        = $self->_primary_key();
+        my $query_string = "SELECT "
+            . $column->name
+            . " FROM "
+            . $self->table
+            . " WHERE $pkey = ?";
+        my $sth = $self->_handle->simple_query( $query_string, $id );
+        my ($value) = eval { $sth->fetchrow_array() };
+        warn $@ if $@;
+
+        $self->{'values'}{ $column->name }  = $value;
+        $self->{'fetched'}{ $column->name } = 1;
+    }
+    if ( $self->{'fetched'}{ $column->name }
+        && !$self->{'decoded'}{ $column->name } )
+    {
+        $self->_apply_output_filters(
+            column    => $column,
+            value_ref => \$self->{'values'}{ $column->name },
+        );
+        $self->{'decoded'}{ $column->name } = 1;
+    }
+
+    return $self->{'values'}{ $column->name };
+}
+
 =head2 _value
 
 _value takes a single column name and returns that column's value for
@@ -520,77 +436,11 @@ sub _value {
     my $column = shift;
 
     my $value = $self->__value( $column => @_ );
-    my $method = $self->can("after_$column");
-    $method->( $self, \$value ) if $method;
+    my $method = "after_$column";
+    $self->$method( \$value ) if ( $self->can($method) );
     return $value;
 }
 
-=head2 __value
-
-Takes a column name and returns that column's value. Subclasses should
-never override __value.
-
-=cut
-
-sub __value {
-    my $self        = shift;
-
-    my $column_name = lc(shift);
-    # If the requested column is actually an alias for another, resolve it.
-    my $column = $self->column($column_name);
-    if  ($column   and defined $column->alias_for_column ) {
-        $column = $self->column($column->alias_for_column());
-        $column_name = $column->name;
-    }
-
-    return unless ($column);
-
-    # In the default case of "yeah, we have a value", return it as
-    # fast as we can.
-    return   $self->{'values'}{$column_name}
-        if ( $self->{'fetched'}{$column_name}
-          && $self->{'decoded'}{$column_name} );
-
-    if ( !$self->{'fetched'}{ $column_name } and my $id = $self->id() ) {
-        my $pkey         = $self->_primary_key();
-        my $query_string = "SELECT "
-            . $column_name
-            . " FROM "
-            . $self->table
-            . " WHERE $pkey = ?";
-        my $sth = $self->_handle->simple_query( $query_string, $id );
-        my ($value) = eval { $sth->fetchrow_array() };
-        warn $@ if $@;
-
-        $self->{'values'}{ $column_name }  = $value;
-        $self->{'fetched'}{ $column_name } = 1;
-    }
-    unless ( $self->{'decoded'}{ $column_name } ) {
-        $self->_apply_output_filters(
-            column    => $column,
-            value_ref => \$self->{'values'}{ $column_name },
-        ) if exists $self->{'values'}{ $column_name };
-        $self->{'decoded'}{ $column_name } = 1;
-    }
-
-    return $self->{'values'}{ $column_name };
-}
-
-=head2 as_hash 
-
-Returns a version of this object's readable columns rendered as a hash of key => value pairs
-
-=cut
-
-sub as_hash {
-    my $self = shift;
-    my %values;
-     map {$values{$_} = $self->$_()} $self->readable_attributes  ;
-     return %values;
-}
-
-
-
 =head2 _set
 
 _set takes a single column name and a single unquoted value.
@@ -609,24 +459,10 @@ sub _set {
     );
 
     my $method = "before_set_" . $args{column};
-    if ( $self->can($method) ) {
-        my $before_set_ret = $self->$method( \%args );
-        return $before_set_ret
-            unless ($before_set_ret);
-    }
+    $self->$method( \%args ) if ( $self->can($method) );
 
-    my $ok = $self->__set(%args);
+    return $self->__set(%args);
 
-    return $ok unless $ok;
-
-    $method = "after_set_" . $args{column};
-    if( $self->can($method) ) {
-        # Fetch the value back to make sure we have the actual value
-        my $value = $self->_value($args{column});
-        $self->$method({column => $args{column}, value => $value});
-    }
-
-    return $ok;
 }
 
 sub __set {
@@ -669,13 +505,7 @@ sub __set {
             )
             || (   defined $args{'value'}
                 && defined $self->{'values'}{ $column->name }
-
-                # XXX: This is a bloody hack to stringify DateTime
-                # and other objects for compares
-                && $args{value}
-                . "" eq ""
-                . $self->{'values'}{ $column->name }
-            )
+                && $args{'value'} eq $self->{'values'}{ $column->name } )
             )
         {
             $ret->as_array( 1, "That is already the current value" );
@@ -683,42 +513,39 @@ sub __set {
         }
     }
 
-    if ( my $sub = $column->validator ) {
-        my ( $ok, $msg ) = $sub->( $self, $args{'value'} );
-        unless ($ok) {
-            $ret->as_array( 0, 'Illegal value for ' . $column->name );
-            $ret->as_error(
-                errno        => 3,
-                do_backtrace => 0,
-                message      => "Illegal value for " . $column->name
-            );
-            return ( $ret->return_value );
-        }
-    }
-    
-
-    # Implement 'is distinct' checking
-    if ( $column->distinct ) {
-        my $ret = $self->is_distinct( $column->name, $args{'value'} );
-        return ( $ret ) if not ( $ret );
+    my $method = "validate_" . $column->name;
+    my ($ok, $msg) = $self->$method( $args{'value'} );
+    unless ( $ok ) {
+        $ret->as_array( 0, 'Illegal value for ' . $column->name );
+        $ret->as_error(
+            errno        => 3,
+            do_backtrace => 0,
+            message      => "Illegal value for " . $column->name
+        );
+        return ( $ret->return_value );
     }
 
-    # The blob handling will destroy $args{'value'}. But we assign
+    # The blob handling will destroy $args{'Value'}. But we assign
     # that back to the object at the end. this works around that
     my $unmunged_value = $args{'value'};
 
-    if ( $column->type =~ /^(text|longtext|clob|blob|lob|bytea)$/i ) {
-        my $bhash = $self->_handle->blob_params( $column->name, $column->type );
-        $bhash->{'value'} = $args{'value'};
-        $args{'value'} = $bhash;
+    unless ( $self->_handle->knows_blobs ) {
+
+        # Support for databases which don't deal with LOBs automatically
+        if ( $column->type =~ /^(text|longtext|clob|blob|lob)$/i ) {
+            my $bhash
+                = $self->_handle->blob_params( $column->name, $column->type );
+            $bhash->{'value'} = $args{'value'};
+            $args{'value'} = $bhash;
+        }
     }
 
     my $val = $self->_handle->update_record_value(
         %args,
         table        => $self->table(),
         primary_keys => { $self->primary_keys() }
-    );
 
+    );
     unless ($val) {
         my $message
             = $column->name . " could not be set to " . $args{'value'} . ".";
@@ -738,25 +565,51 @@ sub __set {
 
         # XXX TODO primary_keys
         $self->load_by_cols( id => $self->id );
-    } else {
-        $self->{'values'}{ $column->name } = $unmunged_value;
+    }
+    else {
+        $self->{'values'}->{ $column->name } = $unmunged_value;
         $self->{'decoded'}{ $column->name } = 0;
     }
     $ret->as_array( 1, "The new value has been set." );
     return ( $ret->return_value );
 }
 
+=head2 _Validate column VALUE
+
+Validate that value will be an acceptable value for column. 
+
+Currently, this routine does nothing whatsoever. 
+
+If it succeeds (which is always the case right now), returns true. Otherwise returns false.
+
+=cut
+
+sub _validate {
+    my $self  = shift;
+    my $column = shift;
+    my $value = shift;
+
+ #Check type of input
+ #If it's null, are nulls permitted?
+ #If it's an int, check the # of bits
+ #If it's a string,
+ #check length
+ #check for nonprintables
+ #If it's a blob, check for length
+ #In an ideal world, if this is a link to another table, check the dependency.
+    return (1);
+}
+
 =head2 load
 
-C<load> can be called as a class or object method.
-
-Takes a single argument, $id. Calls load_by_cols to retrieve the row 
-whose primary key is $id.
+Takes a single argument, $id. Calls load_by_cols to retrieve the row whose primary key
+is $id
 
 =cut
 
 sub load {
     my $self = shift;
+
     return unless @_ and defined $_[0];
 
     return $self->load_by_cols( id => shift );
@@ -764,28 +617,19 @@ sub load {
 
 =head2 load_by_cols
 
-C<load_by_cols> can be called as a class or object method.
-
 Takes a hash of columns and values. Loads the first record that matches all
 keys.
 
 The hash's keys are the columns to look at.
 
 The hash's values are either: scalar values to look for
-OR hash references which contain 'operator' and 'value'
+OR has references which contain 'operator' and 'value'
 
 =cut
 
 sub load_by_cols {
-    my $class    = shift;
+    my $self = shift;
     my %hash = (@_);
-    my ($self);
-    if (ref($class)) {
-            ($self,$class) = ($class,undef);
-    } else {
-            $self = $class->new( handle => (delete $hash{'_handle'} || undef));
-    }
-
     my ( @bind, @phrases );
     foreach my $key ( keys %hash ) {
         if ( defined $hash{$key} && $hash{$key} ne '' ) {
@@ -796,26 +640,23 @@ sub load_by_cols {
                 $op       = $hash{$key}->{operator};
                 $value    = $hash{$key}->{value};
                 $function = $hash{$key}->{function} || "?";
-            } else {
+            }
+            else {
                 $op    = '=';
                 $value = $hash{$key};
             }
 
-            if (ref $value && $value->isa('Jifty::DBI::Record') ) {
-                # XXX TODO: check for proper foriegn keyness here
-                $value = $value->id;
-            }
-
-
             push @phrases, "$key $op $function";
             push @bind,    $value;
-        } else {
+        }
+        else {
             push @phrases, "($key IS NULL OR $key = ?)";
             my $column = $self->column($key);
 
             if ( $column->is_numeric ) {
                 push @bind, 0;
-            } else {
+            }
+            else {
                 push @bind, '';
             }
 
@@ -826,14 +667,11 @@ sub load_by_cols {
         . $self->table
         . " WHERE "
         . join( ' AND ', @phrases );
-    if ($class) { $self->_load_from_sql( $query_string, @bind ); return $self}
-    else {return $self->_load_from_sql( $query_string, @bind );}
-
+    return ( $self->_load_from_sql( $query_string, @bind ) );
 }
 
 =head2 load_by_primary_keys 
 
-Loads records with a given set of primary keys. 
 
 =cut
 
@@ -857,16 +695,8 @@ loaded values hash.
 =cut
 
 sub load_from_hash {
-    my $class    = shift;
+    my $self    = shift;
     my $hashref = shift;
-    my ($self);
-
-    if (ref($class)) {
-            ($self,$class) = ($class,undef);
-    } else {
-            $self = $class->new( handle => (delete $hashref->{'_handle'} || undef));
-    }
-    
 
     foreach my $f ( keys %$hashref ) {
         $self->{'fetched'}{ lc $f } = 1;
@@ -884,9 +714,9 @@ Load a record as the result of an SQL statement
 =cut
 
 sub _load_from_sql {
-    my $self         = shift;
+    my $self        = shift;
     my $query_string = shift;
-    my @bind_values  = (@_);
+    my @bind_values = (@_);
 
     my $sth = $self->_handle->simple_query( $query_string, @bind_values );
 
@@ -915,55 +745,22 @@ sub _load_from_sql {
     foreach my $f ( keys %{ $self->{'values'} } ) {
         $self->{'fetched'}{ lc $f } = 1;
     }
-    return ( 1, "Found object" );
+    return ( 1, "Found Object" );
 
 }
 
-=head2 create PARAMHASH
-
-C<create> can be called as either a class or object method
-
-This method creates a new record with the values specified in the PARAMHASH.
-
-This method calls two hooks in your subclass:
-
-=over
-
-=item before_create
-
-This method is called before trying to create our row in the
-database. It's handed a reference to your paramhash. (That means it
-can modify your parameters on the fly).  C<before_create> returns a
-true or false value. If it returns false, the create is aborted.
-
-=item after_create
-
-This method is called after attempting to insert the record into the
-database. It gets handed a reference to the return value of the
-insert. That'll either be a true value or a L<Class::ReturnValue>
-
-=back
+=head2 create
 
+Takes an array of key-value pairs and drops any keys that aren't known
+as columns for this recordtype
 
 =cut 
 
 sub create {
-    my $class    = shift;
+    my $self    = shift;
     my %attribs = @_;
 
-    my ($self);
-    if (ref($class)) {
-            ($self,$class) = ($class,undef);
-    } else {
-            $self = $class->new( handle => (delete $attribs{'_handle'} || undef));
-    }
-
-
-
-    if ( $self->can('before_create') ) {
-        my $before_ret = $self->before_create( \%attribs );
-        return ($before_ret) unless ($before_ret);
-    }
+    $self->before_create( \%attribs ) if $self->can('before_create');
 
     foreach my $column_name ( keys %attribs ) {
         my $column = $self->column($column_name);
@@ -984,39 +781,23 @@ sub create {
             value_ref => \$attribs{$column_name},
         );
 
-        # Implement 'is distinct' checking
-        if ( $column->distinct ) {
-            my $ret = $self->is_distinct( $column_name, $attribs{$column_name} );
-            return ( $ret ) if not ( $ret );
-        }
+    }
+    unless ( $self->_handle->knows_blobs ) {
 
-        if ( $column->type =~ /^(text|longtext|clob|blob|lob|bytea)$/i ) {
-            my $bhash = $self->_handle->blob_params( $column_name, $column->type );
-            $bhash->{'value'} = $attribs{$column_name};
-            $attribs{$column_name} = $bhash;
+        # Support for databases which don't deal with LOBs automatically
+        foreach my $column_name ( keys %attribs ) {
+            my $column = $self->column($column_name);
+            if ( $column->type =~ /^(text|longtext|clob|blob|lob)$/i ) {
+                my $bhash = $self->_handle->blob_params( $column_name,
+                    $column->type );
+                $bhash->{'value'} = $attribs{$column_name};
+                $attribs{$column_name} = $bhash;
+            }
         }
     }
-
-    for my $column ($self->columns) {
-        if (not defined $attribs{$column->name} and defined $column->default and not ref $column->default) {
-            $attribs{$column->name} = $column->default;
-        }
-        if (not defined $attribs{$column->name} and $column->mandatory and $column->type ne "serial" ) {
-            # Enforce "mandatory"
-            Carp::carp "Did not supply value for mandatory column ".$column->name;
-            return ( 0 );
-        }
-    }
-
     my $ret = $self->_handle->insert( $self->table, %attribs );
-    $self->after_create( \$ret ) if $self->can('after_create');
-    if ($class) {
-        $self->load_by_cols(id => $ret);
-        return ($self);
-    }
-    else {
-     return ($ret);
-    }
+    $self->after_create($ret) if $self->can('after_create');
+    return ($ret);
 }
 
 =head2 delete
@@ -1024,34 +805,11 @@ sub create {
 Delete this record from the database. On failure return a
 Class::ReturnValue with the error. On success, return 1;
 
-This method has two hooks
-
-=over 
-
-=item before_delete
-
-This method is called before the record deletion, if it exists. On
-failure it returns a L<Class::ReturnValue> with the error.  On success
-it returns 1.
-
-If this method returns an error, it causes the delete to abort and return
-the return value from this hook.
-
-=item after_delete
-
-This method is called after deletion, with a reference to the return
-value from the delete operation.
-
-=back
-
 =cut
 
 sub delete {
     my $self = shift;
-    if ( $self->can('before_delete') ) {
-        my $before_ret = $self->before_delete();
-        return $before_ret unless ($before_ret);
-    }
+    $self->before_delete() if $self->can('before_delete');
     my $ret = $self->__delete;
     $self->after_delete( \$ret ) if $self->can('after_delete');
     return ($ret);
@@ -1075,11 +833,12 @@ sub __delete {
 
     $where =~ s/AND\s$//;
     my $query_string = "DELETE FROM " . $self->table . ' ' . $where;
-    my $return       = $self->_handle->simple_query( $query_string, @bind );
+    my $return      = $self->_handle->simple_query( $query_string, @bind );
 
     if ( UNIVERSAL::isa( 'Class::ReturnValue', $return ) ) {
         return ($return);
-    } else {
+    }
+    else {
         return (1);
     }
 }
@@ -1100,27 +859,13 @@ arguably correct.
 
 sub table {
     my $self = shift;
-    $self->TABLE_NAME($self->_guess_table_name) unless ($self->TABLE_NAME());
-    return $self->TABLE_NAME();
-}
 
-=head2 collection_class
+    if ( not ref($self) ) {
+        return $self->_guess_table_name();
+    }
+    $self->{__table_name} ||= $self->_guess_table_name;
 
-Returns the collection class which this record belongs to; override this to
-subclass.  If you haven't specified a collection class, this returns a best
-guess at the name of the collection class for this collection.
-
-It uses a simple heuristic to determine the collection class name -- It
-appends "Collection" to its own name. If you want to name your records
-and collections differently, go right ahead, but don't say we didn't
-warn you.
-
-=cut
-
-sub collection_class {
-    my $self = shift;
-    my $class = ref($self) || $self;
-    $class . 'Collection';
+    return $self->{__table_name};
 }
 
 =head2 _guess_table_name
@@ -1157,13 +902,20 @@ sub _handle {
     return ( $self->{'DBIxHandle'} );
 }
 
-=head2 PRIVATE refers_to
+=for private refers_to
 
 used for the declarative syntax
 
 
 =cut
 
+sub refers_to (@) {
+    my $class = shift;
+    my (%args) = @_;
+
+    return ( refers_to => $class, %args );
+}
+
 sub _filters {
     my $self = shift;
     my %args = ( direction => 'input', column => undef, @_ );
@@ -1199,9 +951,7 @@ sub _apply_filters {
     my $action = $args{'direction'} eq 'output' ? 'decode' : 'encode';
     foreach my $filter_class (@filters) {
         local $UNIVERSAL::require::ERROR;
-        $filter_class->require() unless 
-         $INC{ join('/', split(/::/,$filter_class)).".pm" };
-
+        $filter_class->require();
         if ($UNIVERSAL::require::ERROR) {
             warn $UNIVERSAL::require::ERROR;
             next;
@@ -1217,39 +967,6 @@ sub _apply_filters {
     }
 }
 
-=head2 is_distinct COLUMN_NAME, VALUE
-
-Checks to see if there is already a record in the database where
-COLUMN_NAME equals VALUE.  If no such record exists then the
-COLUMN_NAME and VALUE pair is considered distinct and it returns 1.
-If a value is already present the test is considered to have failed
-and it returns a L<Class::ReturnValue> with the error.
-
-=cut 
-
-sub is_distinct {
-    my $self = shift;
-    my $column = shift;
-    my $value = shift;
-
-    my $record = $self->new( handle => $self->_handle );
-    $record->load_by_cols ( $column => $value );
-
-    my $ret = Class::ReturnValue->new();
-
-    if( $record->id ) {
-        $ret->as_array( 0, "Value already exists for unique column $column");
-        $ret->as_error(
-            errno        => 3,
-            do_backtrace => 0,
-            message      => "Value already exists for unique column $column",
-        );
-        return ( $ret->return_value );
-    } else {
-        return (1);
-    }
-}
-
 1;
 
 __END__
@@ -1258,13 +975,11 @@ __END__
 
 =head1 AUTHOR
 
-Jesse Vincent <jesse@bestpractical.com>, Alex Vandiver <alexmv@bestpractical.com>, David Glasser <glasser@bestpractical.com>, Ruslan Zakirov <ruslan.zakirov@gmail.com>
+Jesse Vincent, <jesse@fsck.com> 
 
-Based on DBIx::SearchBuilder::Record, whose credits read:
+Enhancements by Ivan Kohler, <ivan-rt@420.am>
 
- Jesse Vincent, <jesse@fsck.com> 
- Enhancements by Ivan Kohler, <ivan-rt@420.am>
- Docs by Matt Knopp <mhat@netlag.com>
+Docs by Matt Knopp <mhat@netlag.com>
 
 =head1 SEE ALSO
 
